/*
 * IRC - Internet Relay Chat, ircd/fileio.c
 * Copyright (C) 1998 Thomas Helvey <tomh@inxpress.net>
 * Copyright (C) 1990 Jarkko Oikarinen and
 *		      University of Oulu, Co Center
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "sys.h"
#ifdef _WIN32
#include <io.h>
#endif
#if HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <sys/stat.h>
#include "fileio.h"

#ifdef CR_CHKCONF
#undef RunMalloc
#undef RunFree
#define RunMalloc malloc
#define RunFree free
#else
#include "runmalloc.h"
#endif

aFileBuf* fbopen(const char* filename, const char* mode)
{
  int openmode = 0;
  int pmode = 0;
  aFileBuf* fb = 0;
  int fd;
  /* DEBUG - Sanity check */
  assert(0 != filename && 0 != mode);

  while (*mode)
  {
    switch (*mode)
    {
    case 'r':
      openmode = O_RDONLY;
      break;
    case 'w':
      openmode = O_WRONLY | O_CREAT | O_TRUNC;
      pmode = S_IREAD | S_IWRITE;
      break;
    case 'a':
      openmode = O_WRONLY | O_CREAT | O_APPEND;
      pmode = S_IREAD | S_IWRITE;
      break;
    case '+':
      openmode &= ~(O_RDONLY | O_WRONLY);
      openmode |= O_RDWR;
      break;
    default:
      break;
    }
    ++mode;
  }
#ifdef _WIN32
  openmode |= O_BINARY;
#endif /* _WIN32 */

  /*
   * stop NFS hangs...most systems should be able to open a file in
   * 3 seconds. -avalon (curtesy of wumpus)
   */
  alarm(3);
  if ((fd = open(filename, openmode, pmode)) == -1) {
    alarm(0);
    return 0;
  }
  alarm(0);

  fb = (aFileBuf*) RunMalloc(sizeof(aFileBuf));
  fb->buf = (char*) RunMalloc(BUFSIZ);
  fb->ptr = fb->endp = fb->buf;
  fb->fd = fd;
  fb->pid = 0;
  fb->flags = 0;
  return fb;
}

/*
 * fbopenp - opens a pipe, forks,
 * calls execlp(ename, ename, arg1, arg2, 0),
 * creates a FileBuf from the pipe descriptor, and returns it.
 */
aFileBuf* fbopenp(const char* ename, const char* arg1, const char* arg2)
{
#ifndef _WIN32
  int pi[2];
  int i;
  aFileBuf* fb;
  pid_t pid;

  /* DEBUG - Sanity check */
  assert(0 != ename && 0 != arg1 && 0 != arg2);
  if (pipe(pi) == -1)
    return 0;

  pid = fork();
  switch (pid)
  {
  case -1:
    return 0;
  case 0:
    close(pi[0]);
    if (pi[1] != 1)
    {
      dup2(pi[1], 1);
      close(pi[1]);
    }
    dup2(1, 2);
    for (i = 3; i < MAXCONNECTIONS; ++i)
      close(i);
    execlp(ename, ename, arg1, arg2, 0);
    exit(-1);
  default:
    close(pi[1]);
    fb = (aFileBuf*) RunMalloc(sizeof(aFileBuf));
    fb->buf = (char*) RunMalloc(BUFSIZ);
    fb->ptr = fb->endp = fb->buf;
    fb->pid = pid;
    fb->fd = pi[0];
    fb->flags = 0;
    return fb;
  }
#endif /* !_WIN32 */
  return 0;
}

void fbclose(aFileBuf* fb)
{
  /* DEBUG Sanity check */
  assert(0 != fb);

  close(fb->fd);
  RunFree((void*) fb->buf);
  RunFree((void*) fb);
}

static int fbfill(aFileBuf* fb)
{
  int n = read(fb->fd, fb->buf, BUFSIZ - 1);
  if (n < 0) {
    fb->flags |= FB_FAIL;
    return -1;
  }
  if (0 == n)
    fb->flags |= FB_EOF;
  fb->ptr  = fb->buf;
  fb->endp = fb->buf + n;
  return n;
}

int fbgetc(aFileBuf* fb)
{
  /* DEBUG - Sanity check */
  assert(0 != fb);
  if (fb->ptr < fb->endp)
    return *fb->ptr++;
  if (fb->flags & FB_EOF)
    return EOF;
  return (fbfill(fb) > 0) ? *fb->ptr++ : EOF;
}

char* fbgets(char* buf, size_t len, aFileBuf* fb)
{
  char* p = buf;
  /* DEBUG - Sanity check */
  assert(0 != buf && 0 < len && 0 != fb);

  if (fb->ptr == fb->endp)
  {
    /* buffer empty */
    if (fbfill(fb) <= 0)
      return 0;
  }
  if (fb->ptr < fb->endp) 
  {
    while (--len) 
    {
      *p = *fb->ptr++;
      /* check for newline */
      if ('\n' == *p) 
      {
        *++p = 0;
        return buf;
      }
      /* check for cr */
      else if ('\r' == *p) 
      {
        /* skip newline if there */
        if ((fb->ptr < fb->endp || fbfill(fb) > 0) 
            && '\n' == *fb->ptr)
          ++fb->ptr;
        /* translate to newline */
        *p = '\n';
        *++p = 0;
        return buf;
      }
      ++p;
      if (fb->ptr < fb->endp || fbfill(fb) > 0)
        continue;
      *p = 0;
      return buf;
    }
  }
  return 0;
}
 
int fbputs(const char* str, aFileBuf* fb)
{
  int len;
  /* DEBUG - Sanity check */
  assert(0 != str && 0 != fb);
  len = strlen(str);
  return write(fb->fd, str, len);
}

int fbstat(struct stat* sb, aFileBuf* fb)
{
  /* DEBUG - Sanity check */
  assert(0 != sb && 0 != fb);
  return fstat(fb->fd, sb);
}

int fbpid(aFileBuf* fb)
{
  return fb->pid;
}


