/*
 * IRC - Internet Relay Chat, ports/nt/os/service.c
 * Port library file for NT version of ircd.
 * (c) 1997 Thomas Helvey <tomh@inxpress.net>
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <windows.h>
#include <stdio.h>
#include <string.h>

/* Install options */
#define INSTALL 1
#define REMOVE  2

SC_HANDLE scManager = 0;    /* Handle to service manager */
SC_HANDLE scServ    = 0;    /* Handle to service */

char* versionString = "sinstall version 1.0\n";
LPCTSTR binaryPath = TEXT("%SystemRoot%\\system\\ircd.exe");
LPCTSTR ircdName   = TEXT("ircd");
int installOption = 0;

static void error_exit(const char* msg)
{
  int err = GetLastError();
  if (scServ)
    CloseServiceHandle(scServ);
  if (scManager)
    CloseServiceHandle(scManager);
  fprintf(stderr, "Service installer error: %s (%d) %s\n", 
          ((msg) ? msg : ""), err, strerror(err));
  exit(-1);
}

static void print_usage()
{
  char* usage[] = {
    "sinstall -i <ircd path>        -install ircd",
    "sinstall -f <install file>     -read install options from file",
    "sinstall -d <exename>          -delete ircd",
    "sinstall -h                    -help, shows this screen",
    "sinstall -V                    -Version",
    0
  };
  char** pp = usage;

  fprintf(stderr, "sinstall: Service installer for ircd\n");
  fprintf(stderr, "Usage:\n");
  while (*pp)
    fprintf(stderr, "%s\n", *pp++);
  exit(0);
}

static void load_option_file(const char* opt_path)
{
  if (opt_path) {
    fprintf(stderr, "Read install options from %s\n", opt_path);
    fprintf(stderr, "Not yet implemented, try again later please.\n");
  }
  exit(0);
}

static void do_command_line(int argc, char** argv)
{
  char* p = 0;
  while (--argc >= 0) {
    p = *++argv;
    switch (*p++) {
    case '-':
    case '/':
      break;
    default:
      print_usage();
    }
    switch (*p) {
    case 'i':
      if (argc--) {
        installOption = INSTALL;
        binaryPath = *++argv;
      }
      else
        print_usage();
      break;
    case 'f':
      if (argc--) {
        installOption = INSTALL;
        load_option_file(*++argv);
      }
      else
        print_usage();
      break; 
    case 'd':
      if (argc--) {
        installOption = REMOVE;
        ircdName = *++argv;
      }
      else
        print_usage();
      break;
    case 'h':
      print_usage();
      break;
    case 'V':
      printf("%s\n", versionString);
      exit(0);
    default:
      print_usage();
    }
  }
}

static void create_service()
{
  scServ = CreateService(
        scManager,                    /* SCManager database */
        ircdName,                     /* name of service */
        ircdName,                     /* service name to display */
        SERVICE_ALL_ACCESS,           /* desired access */
        SERVICE_WIN32_OWN_PROCESS,    /* service type */
        SERVICE_DEMAND_START,         /* start type */
        SERVICE_ERROR_NORMAL,         /* error control type */ 
        binaryPath,                   /* service's binary */
        NULL,                         /* load ordering group */
        NULL,                         /* tag identifier */
        NULL,                         /* dependencies */
        NULL,                         /* account (LocalSystem) */
        NULL);                        /* password */
  if (0 == scServ) 
    error_exit("CreateService");
  else 
    printf("CreateService succeded\n");

  CloseServiceHandle(scServ); 
} 

static void delete_service()
{
  SC_HANDLE sc_serv = OpenService( 
        scManager,          /* SCManager database */
        ircdName,           /* name of service */
        DELETE);            /* only need DELETE access */
  if (0 == scServ)
    error_exit("OpenService");  
  if (!DeleteService(scServ))
    error_exit("DeleteService"); 
  else
    printf("DeleteService succeded\n");  
  CloseServiceHandle(scServ);
} 

int main(int argc, char** argv)
{
  if (argc < 2)
    print_usage();
  do_command_line(argc, argv);

  /* 
   * Open a handle to the SC Manager database.
   */
  if ((scManager = OpenSCManager(0, 0, SC_MANAGER_ALL_ACCESS)) == 0) 
    error_exit("OpenSCManager");
  if (INSTALL == installOption)
    create_service();
  else if (REMOVE == installOption)
    delete_service();

  CloseServiceHandle(scManager);
  return 0;
}

